node-loops: Basic, non-blocking loop functions for NodeJS
============================================================================

This project aims to provide a simple syntax for the basic loop operations, allowing them
to be performed in a non-blocking fashion.  Each iteration occurs on a process tick, which
allows other operations, such as timers and socket listeners, to occur during a long loop

## Basics
`var Loops = require('loops');`

This provides the following methods:  

* `Loops.for(init, test, incr, action, [callback])`
* `Loops.while(test, action, [callback])`
* `Loops.do(action, while, [callback])`

---

## Loops.for
Increments a variable until a test condition evaluates to a [falsey][1], passing the current iteration of the variable to an action function.

### Syntax
`Loops.for(init, test, iter, action, [callback]);`

### Parameters
<table>
  <tr>
    <td><code>init</code></td>
    <td>An value or function, of the form <code>function() {...}</code>, to which the internal variable is initially set</td>
  </tr>
  <tr>
    <td><code>test</code></td>
    <td>A sentinal value, or function of the form, <code>function(value) {...}</code>, which determines whether the loop should continue</td>
  </tr>
  <tr>
    <td><code>incr</code></td>
    <td>The value by which the counter is increased at the end of each iteration</td>
  </tr>
  <tr>
    <td><code>action</code></td>
    <td>A function, of the form <code>function(value) {...}</code>, which is performed for each iterated value</td>
  </tr>  
  <tr>
    <td><code>callback</code></td>
    <td>An optional function, of the form <code>function(err, lastValue) {...}</code> which is performed at the end of the iteration (or on first error)</td>
  </tr>  
</table>

### Notes
* If a value is supplied for `init`, this is the initial value for the internal loop variable
* If a function of the form `function() {...}` is supplied for `init`, the function is evaluated and the internal loop value is initialized to its return value
* If a function of the form `function(currentValue) {...}` is supplied for `test`, the function is evaluated for the current loop value; the loop continues if this returns a [truthy][1] result
* If a value of the same type as `init` is supplied for `test`; the loop continues if the current loop value is less than the value supplied
* For all other values supplied for `test`, the loop continues if the value is [truthy][1]; this can lead to infinite loops
* If a value-type is supplied for `incr`, that value is added to the current loop value.  No attempt is made to convert to a common type, it is purely an addition operation.
* If a function of the form `function(currentValue) {...}` is supplied for `incr`, the next loop value is set to the return value of that function.

### Examples
<table border="1">
	<tr>
		<th>Classic JavaScript</th>
		<th>Loops.for</th>
	</tr>
	<tr>
		<td><pre>
var arr = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ];
for (var i = 0, len = arr.length; i < len; i++) {
	console.log('the value of arr[%d] is %d', i, arr[i]);
}
console.log('done => last value of i was %d', i);
		</pre></td>
		<td><pre>
var arr = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ];
Loops.for(0, arr.length, 1, function(i) {
	console.log('the value of arr[%d] is %d', i, arr[i]);
}, function(err, i) {
	console.log('done; last value of i was %d', i);
});
		</pre></td>
	</tr>
	<tr>
		<td><pre>
var arr = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ];
for (var i = 0; i < arr3.length; i++) {
	console.log('the value of arr[%d] is %d', i, ar3[i]);
    arr.pop();
}
console.log('done; last value of i was %d', i);
		</pre></td>
		<td><pre>
var arr = [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ];
Loops.for(0, function(i) { return i < arr.length; }, 1, function(i) {
	console.log('the value of arr[%d] is %d', i, arr[i]);
    arr.pop();
}, function(err, i) {
	console.log('done; last value of i was %d', i);
});
		</pre></td>
	</tr>
</table>

---

## Loops.while
Repeatedly executes an action function while a test condition evaluates to a [truthy][1] value.

### Syntax
`Loops.while(test, action, [callback]);`

### Parameters
<table>
	<tr>
		<td><code>test</code></td>
		<td>A sentinal value, or function of the form, <code>function(value) {...}</code>, which determines whether the loop should continue</td>
	</tr>
	<tr>
		<td><code>action</code></td>
		<td>A function, of the form <code>function() {...}</code> which is executed on each iteration</td>
	</tr>  
	<tr>
		<td><code>callback</code></td>
		<td>An optional function, of the form <code>function(err) {...}</code> which is performed at the end of the iteration (or on first error)</td>
	</tr>  
</table>

### Notes
* If a function of the form `function() {...}` is supplied for `test`, the function is evaluated; the loop continues if this returns a [truthy][1] result
* For all other values supplied for `test`, the loop continues if the value is [truthy][1]; this can lead to infinite loops
* As opposed to `Loops.do`, this will always perform the test before the action; if the test is never [truthy][1], the action will not be performed

### Examples
<table border="1">
	<tr>
		<th>Classic JavaScript</th>
		<th>Loops.while</th>
	</tr>
	<tr>
		<td><pre>
var date1 = (new Date()).getTime();
var date2 = date1 + 5;
while (date1 < date2) {
	console.log('waiting for %d, currently %d', date2, date1);
    date1 = (new Date()).getTime();
}
console.log('done');
		</pre></td>
		<td><pre>
var date1 = (new Date()).getTime();
var date2 = date1 + 5;
Loops.while(function() { return (date1 < date2); }, function() {
    console.log('waiting for %d, currently %d', date2, date1);
    date1 = (new Date()).getTime();
}, function(err) {
    console.log('done');
});
		</pre></td>
	</tr>
</table>

---

## Loops.do
Executes an action function and then repeats it while a test condition evaluates to a [truthy][1] value.

### Syntax
`Loops.do(action, test, [callback]);`

### Parameters
<table>
  <tr>
    <td><code>action</code></td>
    <td>A function, of the form <code>function() {...}</code> which is executed on each iteration</td>
  </tr>  
  <tr>
    <td><code>test</code></td>
    <td>A sentinal value, or function of the form, <code>function(value) {...}</code>, which determines whether the loop should continue</td>
  </tr>
  <tr>
    <td><code>callback</code></td>
    <td>An optional function, of the form <code>function(err) {...}</code> which is performed at the end of the iteration (or on first error)</td>
  </tr>  
</table>

### Notes
* If a function of the form `function() {...}` is supplied for `test`, the function is evaluated; the loop continues if this returns a [truthy][1] result
* For all other values supplied for `test`, the loop continues if the value is [truthy][1]; this can lead to infinite loops
* As opposed to `Loops.while`, this will always perform the action before the test; the action will be performed, at least once, even if the test is never [truthy][1]

### Examples
<table border="1"">
	<tr>
		<th>Classic JavaScript</th>
		<th>Loops.do</th>
	</tr>
	<tr>
		<td><pre>
var date1 = (new Date()).getTime();
var date2 = date1 + 5;
do {
	console.log('waiting for %d, currently %d', date2, date1);
    date1 = (new Date()).getTime();
}
while (date1 < date2);
console.log('done');
		</pre></td>
		<td><pre>
var date1 = (new Date()).getTime();
var date2 = date11 + 5;
Loops.do(function() {
    console.log('waiting for %d, currently %d', date2, date1);
    date1 = (new Date()).getTime();
}, function() { return (date1 < date2); }, function(err) {
    console.log('done');
});
		</pre></td>
	</tr>
</table>

[1]: http://www.sitepoint.com/javascript-truthy-falsy/
